######################## TCL FILE ##############################
# Description
# Command file generator for machine: RFP-6K
# 5 axis cutting Machine Tool
# CNC control system: 4C300
# Software version: 2.0
#
# Revisions
#
#   Date            		Who        		Reason
#  27-mart-2003		                     	    First Release
# $HISTORY$
#
################################################################################
# EVENT HANDLER SECTION

# source "[MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl"
#  MOM_set_debug_mode ON
#_______________________________________________________________________________
#The following command invokes the warning mode and the default settings.
#_______________________________________________________________________________

  source "[MOM_ask_env_var UGII_CAM_POST_DIR]ugpost_base.tcl"

#_______________________________________________________________________________
#The following file consists of default values set for G & M codes
#_______________________________________________________________________________

#---------------- M code declaration

  set mom_sys_program_stop_code                 0
  set mom_sys_optional_stop_code                1
  set mom_sys_end_of_program_code               2

  set mom_sys_spindle_direction_code(NONE)      3
  set mom_sys_spindle_direction_code(CLW)       3
  set mom_sys_spindle_direction_code(CCLW)      4
  set mom_sys_spindle_direction_code(OFF)       5

  set mom_sys_coolant_code(FLOOD)               8
  set mom_sys_coolant_code(ON)                  8
  set mom_sys_coolant_code(OFF)                 9
  set mom_sys_coolant_code(TAP)                 8
  set mom_sys_coolant_code(MIST)                7

  set mom_sys_clamp_code(ON)                    10
  set mom_sys_clamp_code(OFF)                   11
  set mom_sys_clamp_code(AXISON)                45
  set mom_sys_clamp_code(AXISOFF)               46

  set mom_sys_rotary_tool_direction_code(CLW)   13
  set mom_sys_rotary_tool_direction_code(CCLW)  14

  set mom_sys_tailstock_out_code                25
  set mom_sys_tailstock_in_code                 26

  set mom_sys_program_end_code                  2
  set mom_sys_c_axis_on_code                    45
  set mom_sys_c_axis_off_code                   46
  set mom_sys_spindle_brake_clamp_code          68
  set mom_sys_spindle_brake_unclamp_code        69
  set mom_sys_tailstock_clamp_code              78
  set mom_sys_tailstock_unclamp_code            79

#---------------- G code declaration

  set mom_sys_rapid_code                     0
  set mom_sys_linear_code                    1
  set mom_sys_circle_code(CLW)               2
  set mom_sys_circle_code(CCLW)              3

  set mom_sys_delay_code(REVOLUTIONS)        4
  set mom_sys_delay_code(SECONDS)            4

  set mom_sys_polar_mode_on                  12.1
  set mom_sys_polar_mode_off                 13.1

  set mom_sys_cutcom_plane_code(XY)          17
  set mom_sys_cutcom_plane_code(ZX)          18
  set mom_sys_cutcom_plane_code(YZ)          19

  set mom_sys_inch_code                      20
  set mom_sys_metric_code                    21

  set mom_sys_ref_point_return_code          28

  set mom_sys_cutcom_code(OFF)               40
  set mom_sys_cutcom_code(LEFT)              41
  set mom_sys_cutcom_code(RIGHT)             42

  set mom_sys_spindle_max_rpm_code           50
  set mom_sys_wcs_change_code                0

  set mom_sys_mirror_image_on                68
  set mom_sys_mirror_image_off               69

  set mom_sys_cycle_off                      80
  set mom_sys_cycle_drill_code               81
#  G82 is Spot Facing Cycle,C-Bore and C-Sink
  set mom_sys_cycle_drill_dwell_code         82
  set mom_sys_cycle_drill_deep_code          83
  set mom_sys_cycle_breakchip_code           83.1

  set mom_sys_cycle_tap_code                 84
  set mom_sys_cycle_tap_code_left            84.1

  set mom_sys_cycle_bore_code                85
#  G86 is Reaming Cycle
  set mom_sys_cycle_bore_drag_code           86
  set mom_sys_cycle_bore_dwell_code          89

  set mom_sys_spindle_mode_code(SFM)         96
  set mom_sys_spindle_mode_code(SMM)         96
  set mom_sys_spindle_mode_code(RPM)         97

  set mom_sys_feedrate_code(IPM)             98
  set mom_sys_feedrate_code(MMPM)            98
  set mom_sys_feedrate_code(IPR)             99
  set mom_sys_feedrate_code(MMPR)            99

#_______________________________________________________________________________
#
# Kinematic Variable Declaration
#_______________________________________________________________________________

# Note:-  T means ON & F means OFF

  set mom_kin_read_ahead_next_motion	   TRUE
  set mom_kin_machine_type                 5_axis_dual_head
  set mom_kin_linearization_flag           TRUE
  set mom_kin_linearization_tol            0.001
  set mom_kin_coordinate_type              CARTESIAN
  set mom_kin_machine_resolution           0.001
  set mom_kin_max_arc_radius               99999.999
  set mom_kin_min_arc_radius               0.001
  set mom_kin_arc_valid_plane              XY ; #XYZ  XY
  set mom_kin_arc_output_mode              QUADRANT ; # LINEAR
  set mom_kin_rapid_feed_rate              5000.0
  set mom_kin_max_dmp                      7200.0

  #-----------------------------------------------------------------------------
  set mom_kin_4th_axis_plane               YZ
  set mom_kin_4th_axis_direction           MAGNITUDE_DETERIMINES_DIRECTION
  set mom_kin_4th_axis_rotation            reverse ; # standard reverse
  set mom_kin_4th_axis_zero                0.0
  set mom_kin_4th_axis_center_offset(0)    0.0
  set mom_kin_4th_axis_center_offset(1)    0.0
  set mom_kin_4th_axis_center_offset(2)    0.0
  set mom_kin_4th_axis_min_limit           -30.0
  set mom_kin_4th_axis_max_limit           30.0

  set mom_kin_5th_axis_plane               XY
  set mom_kin_5th_axis_direction           MAGNITUDE_DETERIMINES_DIRECTION
  set mom_kin_5th_axis_rotation            standard
  set mom_kin_5th_axis_center_offset(0)    0.0
  set mom_kin_5th_axis_center_offset(1)    0.0
  set mom_kin_5th_axis_center_offset(2)    0.0
  set mom_kin_5th_axis_min_limit           -270.0
  set mom_kin_5th_axis_max_limit           270.0

  set mom_kin_pivot_gauge_offset    0.0

     set mom_kin_4th_axis_min       0
     set mom_kin_5th_axis_min       0

     set mom_kin_4th_axis_max       0
     set mom_kin_5th_axis_max       0

#_______________________________________________________________________________
# Global Variable Declaration for Listing File Output
#_______________________________________________________________________________
         set mom_sys_list_file_columns              132
         set mom_sys_list_file_rows                 80
         set mom_sys_commentary_output              ON
         set mom_sys_list_output                    ON
         set mom_sys_group_output                   OFF
         set mom_sys_header_output                  ON
         set mom_sys_warning_output                 ON
         set mom_sys_rewind_stop_code               %
         set mom_sys_control_in                     ""
         set mom_sys_control_out                    ";"
         set mom_sys_opskip_block_leader            "/"
         set mom_sys_spindle_ranges                 0
         set mom_sys_home_pos(0)                    0
         set mom_sys_home_pos(1)                    0
         set mom_sys_home_pos(2)                    0
         set mom_sys_zero                           0
         set mom_sys_seqnum_start                   1
         set mom_sys_seqnum_incr                    1
         set mom_sys_seqnum_freq                    1
  	 set mom_sys_leader(fourth_axis)            A
  	 set mom_sys_leader(fifth_axis)             C
         set mom_sys_leader(N)                      N

 set mom_event_time                         0
 set mom_work_coordinate_number             0
 set mom_tool_number                        1
 set mom_spindle_rpm                        0
 set mom_tool_gage_length                   0

#-----------------------------------------------------------------------------
#----------------------------- VARIABLES

  set rad2deg [expr 90.0 / asin(1.0)]
  set PI [expr 2.0 * asin(1.0)]                    ; #value PI
  set initial_move 0
  set linear_move 0
  set spindle_speed 0
  set spindle_speed_1 0
  set korr_move  0

#        max  perepad   fedrat (/)
          set    dst(10)         300
          set    dst(11)         300
          set    dst(12)         300

  set output_mode                           "CARTESIAN"
  set yaxis_status                           1
  set count  0
  set operation_flag                         0
  set operation_count                        0
  set rapid_flag                             0
  set g_adjust                               0
  set dinst                                  0
  set dinam                                 29
  set g_torm                                 0
  set j        0
  set a_feed   0
  set nn 0

################################################################################



# ==============   ==========================
#
#       
#   MOM_output_text -      
#
  global buffer          ; #  
  global buffer_output   ; #    
  set count -1           ; # 
  set buffer_action 1   ; # 0 -   ; 1 - 
  set buffer_size -1     ; #   
  set buffer_size_max 3 ; #  
  set buffer_feed_view 3 ; #   
#

#=============================================================
proc PB_feed_buffer { } {
# This proc checks the "buffer" array
# and if it is non empty outputs every string in the "buffer" array.
#=============================================================
  global buffer_size buffer_action
  global buffer
  global buffer_output
  global buffer_param
  global count
  global buffer_feed_view

  if { $buffer_action == 0 } { return }

  global feed
  #   -   1-2-3   
  #    10% (20,30,40)

  set bstep 0 ;  #  

  for { set i [expr $buffer_size-1] } { $i>=0 } { set i [ expr $i -1 ] }  {
 #  MOM_output_literal "i===========$i buffer_size $buffer_size"
    set str $buffer($i)
    set var0 ""
   ; #          = ;%
    if {0!=[regexp -nocase -- {[;]+|[%]+}  $str  var0 ]}  { continue ; }

    set v1 [ expr double($buffer_param($i)) ]
    set v1 [ format "%.1f" $v1 ]
    set v2 [ format "%.1f" $feed ]
    ;#$v1!=0.0 && $v1>$v2
    if {![EQ_is_zero $v1 ] && [ EQ_is_gt $v1 $v2]}  {

      if {$bstep==0} { set value [ expr $v1*0.70 ] }
      if {$bstep==1} { set value [ expr $v1*0.80 ] }
      if {$bstep==2} { set value [ expr $v1*0.90 ] }

      #set mom_warning_info " buffer($i)=$buffer($i) -v1=$v1--v2=$v2-- - buffer_param ($i)= $buffer_param($i)    feed=$feed"
      #MOM_output_to_listing_device $mom_warning_info
      ;#MOM_catch_warning    value=$value

      set value [ format "%.1f" $value ]
      set strv "(F)(\[0-9\])*(\[.\])*(\[0-9\])+"
      if {0!=[regexp -nocase -- "$strv"  $str  var0 ]} {
         ;#     F -    
         regsub -nocase  -- "$strv"  $str  "F$value"  var0
         ;#       
         set buffer_param($i) $value
         set buffer($i) $var0
       } else {
         ;#    F -     
         set buffer_param($i) $value
         append str " F$value"
         set buffer($i) $str
      } ; #   F
      incr bstep
      if {$bstep==$buffer_feed_view} { return $bstep }

     }

 }

 return 0
}


#=============================================================
proc PB_before_output { } {
# This proc do buffer
#=============================================================
  global mom_o_buffer
  global buffer
  global buffer_action  buffer_size
  global buffer_output count
  global buffer_param
  global buffer_size_max

   if { $buffer_action == 1 } {

     incr buffer_size
     set buffer($buffer_size) $mom_o_buffer

     #catch { set buffer_param($buffer_size) [ LIST_PARAM ] ; }

     global feed
     set buffer_param($buffer_size) $feed

     set mom_o_buffer "" ; # catch { unset mom_o_buffer }
     set _tmp [expr $buffer_size_max-1]

     if {[EQ_is_ge $buffer_size $buffer_size_max]} {

         incr count
         set buffer_output($count) $buffer(0)

         ;#    ()
         # outWin=0 -   , 1 - 
         set outWin 0 ;
         if {$outWin==1} {
         	catch { PB_view_buffer }
         }

         ;#  
         for { set k 0 ; } {$k<$buffer_size} { incr k }  {
         	set _tmp [expr $k+1 ] ;
         	set buffer($k) $buffer($_tmp) ;
         	set buffer_param($k) $buffer_param($_tmp) ;
         }
         set buffer($buffer_size) ""
         set buffer_param($buffer_size) 0.0
         incr buffer_size -1

      }

   }

 return 0
}


#=============================================================
proc LIST_PARAM { } {
# lindex $name_list $index
# linsert $name_list $index|end element ?element element ...?
# lappend varName ?value value value ...?
# llength list
# lreplace list first|last ?element element
#=============================================================
 global mom_warning_info
 global mom_motion_event mom_motion_type  mom_current_motion
 global mom_pos_arc_center  mom_arc_angle
 global mom_pos com_feed_rate mom_spindle_speed
 global mom_motion_distance
 global feed
 global buffer_action

 set outs [ list "none" ]
 if { $buffer_action == 1 } {
    set p0 [format "%8.4f"  $mom_pos(0) ]
    set p1 [format "%8.4f"  $mom_pos(1) ]
    set p2 [format "%8.4f"  $mom_pos(2) ]
    set ff [format "%4.2f"  $feed ]
    set ss [format "%4.2f"  $mom_spindle_speed]
    set outs [ list $p0 $p1 $p2 $ff $ss ]
    ;#
    set dd 0.0
    if {[info exists mom_motion_distance]} {
    	 set dd [format "%8.4f"  $mom_motion_distance ]
    }
    set outs [linsert $outs end $dd ]
    ;#
    if {[info exists mom_current_motion]} {
      if { $mom_current_motion=="circular_move"} {
      	set p0 [format "%8.4f"  $mom_pos_arc_center(0) ]
      	set p1 [format "%8.4f"  $mom_pos_arc_center(1) ]
      	set p2 [format "%8.4f"  $mom_pos_arc_center(2) ]
      	set aa [format "%8.4f"  $mom_arc_angle ]
      	set outs [linsert $outs end $p0 $p1 $p2 $aa ]
      }
      set ss  [format "%s"  $mom_current_motion]
      set outs [linsert $outs end $ss ]
    } else {
    	set outs [linsert $outs end {none} ]
    }
    set ss "none"
    if {[info exists mom_motion_type]} {
    	 set ss  [format "%s"  $mom_motion_type]
    }
    set outs [ linsert $outs end $ss ]
  }

 return $outs
}


#=============================================================
proc PB_flush_buffer { } {
# This proc checks the "buffer" array and if it is non empty outputs every string in the "buffer" array.
#=============================================================
  global buffer_size buffer_action
  global buffer
  global buffer_output
  global count
  global TIME

;# MOM_output_to_listing_device "buffer_size=$buffer_size"
;# MOM_output_to_listing_device "buffer_action=$buffer_action"

 if { $buffer_action == 0 } { return }

 MOM_set_seq_off

 if { $buffer_action == 1 } {
	for {set k 0} {$k <= $buffer_size} {incr k}  {
 		incr count
 		set buffer_output($count) $buffer($k)
    	}

  set buffer_action 0
  if { $buffer_size > 0 } {
     #MOM_set_seq_on

     for {set k 0} {$k <= $count} {incr k}  {
     	#set st 0 ; set 	var0 "" ;
     	#set st [regexp -nocase -- {(%)+|([(])+|([)])+|(;)+|(\$)+}  $buffer_output($k)  var0]
     	#if {$st==0} {
     	  if {$buffer_output($k)==""}  { ; }  else  { MOM_output_literal $buffer_output($k) ; }
     	 #} else {
     	  # MOM_set_seq_off
     	  # MOM_output_literal $buffer_output($k) ;
     	  # MOM_set_seq_on
     	 #}

     }

    #MOM_set_seq_off
    set buffer_size 0

    #set _tmp [expr $TIME/60.0 ]
    #MOM_output_to_listing_device "   \n\t TIME=$TIME (sec) \n\t TIME=[ format "%.0f" $_tmp ] ()"
  }

 }

}


#=============================================================
proc  PB_view_buffer {} {
#       !   !
#=============================================================
  global buffer
  global buffer_action  buffer_size
  global buffer_size_max
  global buffer_param

  #global  mom_event_handler_file_name
  #set path [file dirname $mom_event_handler_file_name]

  ; #    #UGII_TMP_DIR=${TMP}
  set path [MOM_ask_env_var TEMP]
  if {$path==""} { set path [MOM_ask_env_var TMP] }
  if {$path==""} { set path [MOM_ask_env_var UGII_TMP_DIR ] }
  if {$path==""} { ;
   } else {
     set strMes [file join $path "buftcl.tcl" ]
     if {[file exists $strMes ]==0} {
     	catch { set wf [ open $strMes w ] ; } errMes ; MOM_log_message "PB_view_buffer-> =$strMes =$errMes" ;
 	puts $wf "\n if \{\$argc!=0\} \{"
 	puts $wf "set strA \[lindex \$argv 0 \]" ;
 	puts $wf "after idle \{.dialog1.msg configure -wraplength 10i\}" ;
 	puts $wf "set i \[tk_dialog .dialog1 \"Buffer view\" \"\$strA\" info 0 OK \]" ;
 	puts $wf "#tk_messageBox -message \"\$strA\" -icon error -title \"......\" " ;
 	puts $wf "\} else \{ " ;
 	puts $wf "  tk_messageBox -message \"  .\" -icon info -title \" ...\" " ;
 	puts $wf "\}" ;
 	puts $wf "exit ;" ;
 	close $wf
     }
     if {[file exists $strMes ]} {
 	; # source $strMes
 	set var0 "" ;
  	set _tmp [expr $buffer_size_max-1]
  	for {set j 0;} {$j<$buffer_size} { incr j }  { set var0 "$var0$buffer($j);  =  $buffer_param($j)\n" ; }
 	set my_wish "[MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]ugwish.exe"
 	exec $my_wish $strMes -- $var0
      }
    }

 return 0
}

# ==============   ==========================




proc MOM_msys {} {
#_______________________________________________________________________________
#
# This procedure is called for a MSYS statement
#_______________________________________________________________________________
}


proc MOM_from_move {} {
#_______________________________________________________________________________
#
# This procedure is called for a FROM statement
#_______________________________________________________________________________
  MOM_do_template from CREATE
}

#=============================================================
proc PB_approach_move { } {
#=============================================================
}


#=============================================================
proc PB_engage_move { } {
#=============================================================
}


#=============================================================
proc PB_first_cut { } {
#=============================================================
}

#=============================================================
proc PB_retract_move { } {
#=============================================================
 #catch { PB_dinam_output }
}

#=============================================================
proc PB_return_move { } {
#=============================================================
  #catch { PB_dinam_output }
}

#=============================================================
proc PB_departure_move { } {
#=============================================================
 #catch { PB_dinam_output }
}

proc MOM_gohome_move {} {
#_______________________________________________________________________________
#
# This procedure is called for a GOHOME move
#_______________________________________________________________________________
  catch { PB_dinam_output }
  MOM_do_template linear
#  MOM_do_template gohome
}

#=============================================================
proc PB_dinam_output { } {
#=============================================================
  global dinam
  set   dinam   29
#  MOM_force once G_torm_29
}


proc MOM_start_of_program {} {
#=============================================================
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
#=============================================================
  global mom_output_file_basename
  global mom_warning_info
  global mom_logname mom_date mom_machine_name  mom_part_name
  global mom_coolant_status mom_coolant_mode
  global mom_cutcom_status mom_cutcom_mode mom_cutcom_plane
  global mom_clamp_status mom_cycle_status
  global mom_spindle_status
  global mom_cutcom_adjust_register mom_tool_adjust_register
  global mom_tool_length_adjust_register mom_length_comp_register
  global mom_flush_register mom_wire_cutcom_adjust_register
  global pb_start_of_program_flag  is_from

#---- Unsetting default setting for variables

    set is_from ""
    set pb_start_of_program_flag 0
    set mom_coolant_status UNDEFINED
    set mom_cutcom_status  UNDEFINED
    set mom_clamp_status   UNDEFINED
    set mom_cycle_status   UNDEFINED
    set mom_spindle_status UNDEFINED
    set mom_cutcom_plane   UNDEFINED

  catch {unset mom_coolant_mode}
  catch {unset mom_coolant_status}
  catch {unset mom_cutcom_mode}
  catch {unset mom_cutcom_status}

    catch {unset mom_cutcom_adjust_register}
    catch {unset mom_tool_adjust_register}
    catch {unset mom_tool_length_adjust_register}
    catch {unset mom_length_comp_register}
    catch {unset mom_flush_register}
    catch {unset mom_wire_cutcom_adjust_register}

  OPEN_files ; #------- The following procedure opens the warning and listing files

  LIST_FILE_HEADER  ; #------- The following procedure lists the header information in commentary data

}

#=============================================================
proc PB_start_of_program { } {
#=============================================================
  global mom_output_file_basename
  global mom_warning_info
  global mom_logname mom_date mom_machine_name  mom_part_name

#------- These are to be obtained for the user before Postprocessing

  set str "\n   ' CC 300 M (4C300)' (5x- )"
  MOM_output_to_listing_device $str

  set str "\n :\n"
  append str "\tG01 -  \n"
  append str "\tG07 -   \n"
  append str "\tG08 -     \n"
  append str "\tG15 -  KV- \n"
  append str "\tG29 -    \n"
  append str "\tG67 -    \n"
  append str "\tG62 -   \n"
  append str "\tG66 -    100% \n"
  append str "\tG66 -   (  )\n"
  append str "\tG79 -      \n"
  append str "\tG99 -   \n"
  append str "\tG145 -     \n"
  append str "\tG17 -       XY\n"
  append str "\tG38 -   , ,  \n"
  append str "\tG68 -      \n"
  append str "\tG71 -    \n"
  append str "\tG90 -  \n"
  append str "\tG94 -    /\n"
  append str "\tG40 -    \n"
  append str "\tG50 -   \n"
  MOM_output_to_listing_device $str
  
#  MOM_set_seq_off
#  MOM_output_literal "; $mom_output_file_basename"
#  MOM_output_literal "; $mom_logname, Date:$mom_date"
#  MOM_output_literal "; MACHINE: $mom_machine_name"
#  MOM_output_literal "   "
#  MOM_do_template rewind_stop
#  MOM_set_seq_on
  MOM_output_literal "(UAO,2)"
  MOM_output_literal "G90G0XYAC"
}


proc MOM_end_of_program {} {
#_______________________________________________________________________________
# This procedure is executed at the end of the program after all
# the paths are traversed.
#_______________________________________________________________________________
 global mom_tool_number
 global nn warning_1
   global  list_file b1 j
  global  mom_kin_4th_axis_min mom_kin_5th_axis_min
  global  mom_kin_4th_axis_max mom_kin_5th_axis_max
  MOM_output_literal "T0.0"
  MOM_output_literal "(UAO,0)"
  MOM_output_literal "G0G90XYZAC"
  MOM_set_seq_off
    	     for {set l 1} {$l <= $nn} {incr l} {
    	MOM_output_literal "$warning_1($l)"
}
  MOM_set_seq_on
  MOM_do_template end_of_program

#---- The following procedure lists the tool list with time in commentary data

       set j  [ expr ($j+1 ) ]
       for {set l 1} {$l < $j} {incr l} {
#          MOM_output_to_listing_device "   b1  $l $b1($l)"
       }

  set  mom_kin_4th_axis_min [format "%2.3f" $mom_kin_4th_axis_min ]
  set  mom_kin_4th_axis_max [format "%2.3f" $mom_kin_4th_axis_max ]
  set  mom_kin_5th_axis_min [format "%2.3f" $mom_kin_5th_axis_min ]
  set  mom_kin_5th_axis_max [format "%2.3f" $mom_kin_5th_axis_max ]

  puts $list_file  "     \n\n\n\n\n\n\n       "
  puts $list_file  "  COORDINATE AXIS TRAVEL - MINIMUM       MAXIMUM         TOTAL   "
  puts $list_file  "           X                                                                "
  puts $list_file  "           Y                                                                "
  puts $list_file  "           Z                                                                "
  puts $list_file  "           A              $mom_kin_4th_axis_min        $mom_kin_4th_axis_max      "
  puts $list_file  "           B              $mom_kin_5th_axis_min        $mom_kin_5th_axis_max       "
  puts $list_file  "     \n\n\n\n\n\n\n  "

  catch { PB_flush_buffer }

  LIST_FILE_TRAILER

#---- The following procedure closes the warning and listing files

  CLOSE_files
}



proc MOM_start_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of each path in the task
# It gets called after all the general parameters of the path are loaded
# like the path name and the tool name.
#_______________________________________________________________________________
  global pb_start_of_program_flag
  if {$pb_start_of_program_flag == 0} {
	PB_start_of_program
     	set pb_start_of_program_flag 1
   }
  global mom_tool_name
  global mom_path_name
  MOM_force once G_motion F
}


proc MOM_end_of_path {} {
#_______________________________________________________________________________
# This procedure is executed at the end of a path.
#_______________________________________________________________________________

}

proc MOM_tool_change {} {
#_______________________________________________________________________________
# This procedure is executed at any time there is a tool change.
# It gets called after the tool change data is loaded like the tool number
# and the register numbers.
#_______________________________________________________________________________

  global mom_path_name
  global mom_tool_name
  global mom_tool_number
  global mom_tool_adjust_register
  global mom_operation_type

}

proc MOM_first_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the first operation, with a tool, in the task.
#_______________________________________________________________________________

  global mom_path_name
  global mom_tool_name
  global mom_tool_number
  global mom_tool_adjust_register
  global mom_operation_type

}


proc MOM_initial_move {} {
#_______________________________________________________________________________
# This procedure is executed for the initial move of each operation.  It
# assumes the tool is moving from a safe position at rapid to the start of
# the operation.
#_______________________________________________________________________________
  global mom_coolant_mode mom_coolant_status
  global mom_motion_type

  global initial_move
  set initial_move 1

#------- These are New Tool Start blocks:- Move to Tool Change Positon

#  MOM_force once  G_motion X Y Z M_cool
  if {[info exists mom_coolant_mode]} { set mom_coolant_status $mom_coolant_mode }

#  
   MOM_force once  G_motion X Y Z
   MOM_do_template rapid_xyz
   global dinam
   set   dinam   28
   MOM_force once G_torm_29
}


##########################################################################################
#_______________________________________________________________________________
proc MOM_first_move {} {
#_______________________________________________________________________________
  global mom_motion_event
  global mom_coolant_mode mom_coolant_status
  if {[info exists mom_coolant_mode]} { set mom_coolant_status $mom_coolant_mode }
  catch {MOM_$mom_motion_event}
}


proc MOM_before_motion {} {
#_______________________________________________________________________________
  global mom_motion_event mom_motion_type  mom_current_motion
  global mom_warning_info mom_seqnum warning_1 nn
  FEEDRATE_SET

  global g_plane  ;  set g_plane [CHECK_PLANE]

    switch $mom_motion_type {
      ENGAGE   {PB_engage_move}
      APPROACH {PB_approach_move}
      FIRSTCUT {PB_first_cut}
      RETURN   {PB_return_move}
      RETRACT  {PB_retract_move}
      DEPARTURE {PB_departure_move}
    }
#           ( +-270 .)
#          ( +-30 .)

 	   if {$mom_warning_info== "unable to determine valid rotary positions" } {
   incr nn
 set warning_1($nn) [format "; :   %.0f      (+-30 )" $mom_seqnum ]
   set mom_warning_info ""
	  }
   if {$mom_warning_info== "ROTARY CROSSING LIMIT."   } {
   incr nn
 set warning_1($nn) [format "; :   %.0f      C" $mom_seqnum ]
   set mom_warning_info ""
}
#*************************************************

    if { [llength [info commands PB_CMD_before_motion]] }     { PB_CMD_before_motion }
    if { [llength [info commands PB_CMD_kin_before_motion]] } { PB_CMD_kin_before_motion }
}


#=============================================================
#  VEC3_init {x y z w}             w = {x y z }         Vector init
proc  VEC3_init {x y z w} {
#=============================================================
      set er ""
      catch { upvar $x x1 } er ; if {$er!=""} { set er "" ; set x1 $x }
      catch { upvar $y y1 } er ; if {$er!=""} { set er "" ; set y1 $y }
      catch { upvar $z z1 } er ; if {$er!=""} { set er "" ; set z1 $z }
      upvar $w w1
      set w1(0) $x1 ; set w1(1) $y1 ; set w1(2) $z1
      return 0
}


VEC3_init 0.0 0.0 0.0 pos_perepad
VEC3_init 0.0 0.0 0.0 nxt_pos_perepad

#===========================================================
proc PB_CMD_before_motion { } {
#===========================================================

  global mom_prev_pos mom_pos mom_tool_axis mom_motion_distance
  global mom_motion_type
  global mom_current_motion
  global g_plane
  global feed  mom_kin_rapid_feed_rate
  global mom_nxt_feed_rate mom_nxt_motion_distance mom_nxt_motion_event mom_nxt_tool_axis mom_nxt_mcs_goto
  global dst pos_perepad nxt_pos_perepad g_torm

#MOM_output_literal "mom_tool_axis $mom_tool_axis(0) $mom_tool_axis(1) $mom_tool_axis(2) "
#MOM_output_literal "mom_nxt_tool_axis $mom_nxt_tool_axis(0) $mom_nxt_tool_axis(1) $mom_nxt_tool_axis(2) "
#MOM_output_literal "mom_nxt_mcs_goto $mom_nxt_mcs_goto(0) $mom_nxt_mcs_goto(1) $mom_nxt_mcs_goto(2) "
#tmin=s/v
    set   g_torm  0
  if {[EQ_is_ge $mom_motion_distance 0.005]} {
    set tmin [expr ( $mom_motion_distance / $feed )]
    #MOM_output_literal "tmin=$tmin mom_motion_distance=$mom_motion_distance   feed=$feed "
    VEC3_sub mom_pos mom_prev_pos vec
    set tmp [ expr 1.0/$tmin ]
    VEC3_scale tmp vec pos_perepad
    if { $mom_nxt_motion_event=="linear_move" || \
         $mom_nxt_motion_event=="rapid_move" } {
              VEC3_sub mom_nxt_mcs_goto mom_pos w1
              set  nxt_motion_distance [ VEC3_mag w1 ]
              if {[EQ_is_ge $nxt_motion_distance 0.01]} {
                 if {[ EQ_is_zero $mom_nxt_feed_rate]} { set mom_nxt_feed_rate $mom_kin_rapid_feed_rate }
                 set tmin1 [expr $nxt_motion_distance / $mom_nxt_feed_rate ]
                 #MOM_output_literal "tmin1=$tmin1"
                 set tmp [ expr 1.0/$tmin1 ]
                 VEC3_sub mom_nxt_mcs_goto mom_pos vec
                 VEC3_scale tmp vec nxt_pos_perepad
               } else {
               	for {set ii 0} {$ii < 3} {incr ii} { set nxt_pos_perepad($ii) $pos_perepad($ii) }
               }
      } else {
        for {set ii 0} {$ii < 3} {incr ii} { set nxt_pos_perepad($ii) $pos_perepad($ii) }
    }

    VEC3_sub nxt_pos_perepad pos_perepad ss
    for {set ii 0} {$ii < 3} {incr ii} { set ss($ii) [expr abs($ss($ii)) ] }
    # MOM_output_literal "ss1  ==  $ss1  ss2 == $ss2   ss3 == $ss3 "

    #  insert  G09
    if {($ss(0) > $dst(10)||$ss(1) > $dst(11)||$ss(2) > $dst(12)&&($ss(0) != 0||$ss(1) != 0||$ss(2) != 0)&& \
       ($mom_current_motion=="linear_move") && \
        ($mom_current_motion=="linear_move") || ($mom_current_motion=="rapid_move"))} {
  		MOM_force once G_torm
  		set  g_torm          9
         }
   }

}
#===========================================================
proc PB_CMD_kin_before_motion  { } {
#===========================================================

  global mom_prev_pos mom_pos mom_tool_axis mom_motion_distance mom_nxt_pos
  global mom_motion_type
  global mom_current_motion
  global g_plane
  global feed  mom_kin_rapid_feed_rate
  global buffer_output count a_feed
  global mom_nxt_feed_rate mom_nxt_motion_distance mom_nxt_motion_event mom_nxt_tool_axis mom_nxt_mcs_goto
  global dst pos_perepad nxt_pos_perepad const_feed g_torm j b1 a_feed
    set const_feed   $feed

#MOM_output_literal "mom_tool_axis $mom_tool_axis(0) $mom_tool_axis(1) $mom_tool_axis(2) "
#MOM_output_literal "mom_nxt_tool_axis $mom_nxt_tool_axis(0) $mom_nxt_tool_axis(1) $mom_nxt_tool_axis(2) "
  set  a [ expr abs($mom_nxt_mcs_goto(0)-$mom_pos(0))]
  if {[EQ_is_le $a 2.0]&&![EQ_is_equal [format "%.2f" $mom_nxt_mcs_goto(0)] [format "%.2f" $mom_pos(0)] ]} {
#MOM_output_literal "mom_prev_pos $mom_prev_pos(0) $mom_prev_pos(1) $mom_prev_pos(2) "
#MOM_output_literal "mom_pos $mom_pos(0) $mom_pos(1) $mom_pos(2) "
#MOM_output_literal "mom_nxt_mcs_goto $mom_nxt_mcs_goto(0) $mom_nxt_mcs_goto(1) $mom_nxt_mcs_goto(2) "

   set b2 $g_torm

   if {$b2 == 0} {
     set feed [ expr ($const_feed*0.3) ]
     if {$feed != $a_feed} {
        set  b_feed [ expr ($const_feed*0.5) ]
        set  c_feed [ expr ($const_feed*0.7) ]
        set count_1 [ expr ($count)]
#        append buffer_output($count_1) "F$b_feed"
        set count_1 [ expr ($count+1)]
#        append buffer_output($count_1) "F$c_feed"
     }

     catch { PB_feed_buffer }

     set a_feed  $feed
     incr  j +1
     set b1($j) [ expr ([ MOM_ask_address_value N]+1)]
     #MOM_output_literal " count $count  buffer_output $buffer_output($count)"
  }
 }

}


proc MOM_linear_move {} {
#_______________________________________________________________________________
# This procedure is executed for each linear move. It gets called after
# the positioning varables, position and tool axis (X,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________

  global mom_cutcom_mode  mom_cutcom_status
  global output_mode
  global linear_move mom_out_angle_pos
  global  mom_kin_4th_axis_min mom_kin_5th_axis_min
  global  mom_kin_4th_axis_max mom_kin_5th_axis_max const_feed

  set initial_move 1

  if {[info exists mom_cutcom_mode]} { set mom_cutcom_status $mom_cutcom_mode }

  if {$mom_out_angle_pos(0) > 0} {
    if {$mom_out_angle_pos(0) > $mom_kin_4th_axis_max} {  set mom_kin_4th_axis_max $mom_out_angle_pos(0) }
  } elseif {$mom_out_angle_pos(0) < 0} {
          if {$mom_out_angle_pos(0) < $mom_kin_4th_axis_min} { set mom_kin_4th_axis_min $mom_out_angle_pos(0) }
  }

  if {$mom_out_angle_pos(1) > 0} {
    if {$mom_out_angle_pos(1) > $mom_kin_5th_axis_max} { set mom_kin_5th_axis_max $mom_out_angle_pos(1) }
  } elseif {$mom_out_angle_pos(1) < 0} {
          if {$mom_out_angle_pos(1) < $mom_kin_5th_axis_min} { set mom_kin_5th_axis_min $mom_out_angle_pos(1) }
  }

# MOM_output_literal " 4th_axis_min=$mom_kin_4th_axis_min  5th_axis_min=$mom_kin_5th_axis_min  "
  MOM_do_template linear
     set feed  $const_feed
}


proc MOM_circular_move {} {
#_______________________________________________________________________________
# This procedure is executed for each circular move. It gets called after
# the circle data are loaded (end point and center).
#_______________________________________________________________________________
  global mom_pos_arc_axis
  global mom_pos mom_prev_pos
  global mom_path_name

  global g_plane
  global mom_cutcom_mode mom_cutcom_status
  global mom_motion_distance mom_arc_radius PI
  global operation

  switch $g_plane {
     "17"
        {
          MOM_suppress once K
          MOM_force always  I J
          VEC3_sub mom_pos mom_prev_pos w
          if { [expr abs($w(0))]<0.01 && [expr abs($w(1))]<0.01 } {
             set f [ expr 2.0*$PI*$mom_arc_radius ]
             #MOM_output_literal "motion_distance=$mom_motion_distance f=$f "
             set _tmp [ expr ( $f-$mom_motion_distance ) ]
             set _tmp [ format "%-.7f" $_tmp ]
             if {![EQ_is_zero $_tmp]} {
               if { [expr abs($_tmp)]>0.1 } { MOM_linear_move ; return ; }
             }
          }
        }
     "18"
        {
          MOM_suppress once J
          MOM_output_to_listing_device "WARNING: Path :- $mom_path_name"
          MOM_output_to_listing_device "WARNING: Cannot do circles in G18 plane"
        }
     "19"
        {
          MOM_output_to_listing_device "WARNING: Path :- $mom_path_name"
          MOM_output_to_listing_device "WARNING: Cannot do circles in G19 plane"
        }
  }

  if {[info exists mom_cutcom_mode]} { set mom_cutcom_status $mom_cutcom_mode  }

#  MOM_do_template plane
  MOM_do_template circle
}


proc MOM_rapid_move {} {
#_______________________________________________________________________________
# This procedure is executed for each rapid move. It gets called after
# the positioning varables, position and tool axis (X,Z,TAX,TAY,TAZ) are loaded.
#_______________________________________________________________________________
  global mom_coolant_mode
  global mom_coolant_status
  global mom_motion_type

  global feed
  global output_mode

  if {[info exists mom_coolant_mode]} { set mom_coolant_status $mom_coolant_mode }

  if { $mom_motion_type != "CYCLE" } {
           MOM_do_template rapid_xy
           MOM_do_template rapid_z
  } else {
#       MOM_do_template rapid_c
       MOM_do_template rapid_xy
       MOM_do_template rapid_z
  }
}



proc  cycle_set {} {
#_______________________________________________________________________________
# This procedure is executed for each new cycle.  This will make sure all
# non-modal addresses and Cycle parameters are output that need to be output
# at the start of a new cycle.
#_______________________________________________________________________________
  global mom_cycle_retract_mode
  global g_return

  MOM_force once G_motion X Z R cycle_dwell cycle_nodrag cycle_step

  if {$mom_cycle_retract_mode == "MANUAL"} {
      set g_return 99
  } elseif {$mom_cycle_retract_mode == "AUTO"} {
      set g_return 98
  } else {
      set g_return 99
  }
}

proc MOM_cycle_plane_change {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion when there is a cycle
# move occuring at a higher level than the previous position.
#_______________________________________________________________________________
  MOM_do_template cycle_plane_change
}

proc MOM_cycle_off {} {
#_______________________________________________________________________________
# This procedure is executed at the end of cycle motion.
#_______________________________________________________________________________
  MOM_set_address_format G_motion Register
  MOM_do_template cycle_off
}

proc MOM_drill {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill cycle.
#_______________________________________________________________________________
  cycle_set
}

proc MOM_drill_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_drill
}

proc MOM_drill_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill dwell cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_drill_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill dwell cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_drill_dwell
}

proc MOM_drill_counter_sink {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill cycle.
#_______________________________________________________________________________
         MOM_drill
}

proc MOM_drill_counter_sink_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill cycle.
#_______________________________________________________________________________
  MOM_drill_move
}

proc MOM_drill_csink_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a drill dwell cycle.
#_______________________________________________________________________________
	 MOM_drill_dwell
}

proc MOM_drill_csink_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a drill dwell cycle.
#_______________________________________________________________________________
  MOM_drill_dwell_move
}

proc MOM_drill_deep {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a deep cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_drill_deep_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a  deep cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_drill_deep
}

proc MOM_drill_break_chip {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a brkchp cycle.
#_______________________________________________________________________________
       	 cycle_set
}

proc MOM_drill_break_chip_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a  brkchp cycle.
#_______________________________________________________________________________
  MOM_set_address_format G_motion Coordinate
  MOM_do_template cycle_breakchip
}

proc MOM_tap {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a tap cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_tap_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a tap cycle.
#_______________________________________________________________________________

  global mom_spindle_direction

  global cycle_tap_code
  global mom_sys_cycle_tap_code_left
  global mom_sys_cycle_tap_code

  if {$mom_spindle_direction == "CCLW"} {
      MOM_set_address_format G_motion Coordinate
      set cycle_tap_code $mom_sys_cycle_tap_code_left
  } else {
      set cycle_tap_code $mom_sys_cycle_tap_code
  }
  MOM_do_template cycle_tap
}

proc MOM_bore {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore
}

proc MOM_bore_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore dwell cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_dwell
}

proc MOM_bore_drag {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore drag cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_drag_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore_drag cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_drag
}

proc MOM_bore_no_drag {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore nodrag cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_no_drag_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a bore nodrag cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_no_drag
}

proc MOM_bore_back {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore back cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_back_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a back bore cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_back
}

proc MOM_bore_manual {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore manual cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_manual_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a manual bore cycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_manual
}

proc MOM_bore_manual_dwell {} {
#_______________________________________________________________________________
# This procedure is executed at the begining of a bore manual dwell bore cycle.
#_______________________________________________________________________________
	 cycle_set
}

proc MOM_bore_manual_dwell_move {} {
#_______________________________________________________________________________
# This procedure is executed for the goto of a manual dwell borecycle.
#_______________________________________________________________________________
  MOM_do_template cycle_bore_manual_dwell
}

proc MOM_load_tool {} {
#_______________________________________________________________________________
# This procedure is executed for the LOAD/TOOL command.  All output for the LOAD
# command will take place with the initial move.
#_______________________________________________________________________________
   global  korr_move
   set  korr_move   1
#  MOM_force once  Z
  MOM_do_template  tool_change

#  MOM_output_literal "(UAO,1)"
}

proc MOM_sequence_number {} {
#_______________________________________________________________________________
# This procedure is executed for the Sequence Number command.
#_______________________________________________________________________________
    SEQNO_SET
}

proc MOM_set_modes {} {
#_______________________________________________________________________________
# This procedure is executed for the Set Mode command.
#_______________________________________________________________________________
  global mom_output_mode
  global mom_arc_mode
  global mom_feed_rate_output_mode
  global mom_parallel_to_axis
  global mom_machine_mode
  global mom_modes_text
  global g_output_mode

# "ABSOLUTE", "INCREMENTAL"
  if {$mom_output_mode == "ABSOLUTE"} {
      MOM_incremental OFF X Z Y
  } elseif {$mom_output_mode == "INCREMENTAL"} {
      MOM_incremental ON X Z Y
  }

# "LINEAR", "CIRCULAR"

  if {$mom_arc_mode == "LINEAR"} {
  } elseif {$mom_arc_mode == "CIRCULAR"} {
  }

# "OFF", "IPM", "IPR", "MMPM", "MMPR", "INVERSE"

  if {$mom_feed_rate_output_mode == "FPM"} {
  } elseif {$mom_feed_rate_output_mode == "FPR"} {
  } elseif {$mom_feed_rate_output_mode == "INVERSE"} {
  } elseif {$mom_feed_rate_output_mode == "OFF"} {
  }

# "ZAXIS", "WAXIS", "UAXIS"

  if {$mom_parallel_to_axis == "ZAXIS"} {
  } elseif {$mom_parallel_to_axis == "WAXIS"} {
  } elseif {$mom_parallel_to_axis == "UAXIS"} {
  }

# "MILL", "TURN", "PUNCH", "LASER", "TORCH", "WIRE"

  if {$mom_machine_mode == "MILL"} {
  } elseif {$mom_machine_mode == "TURN"} {
  } elseif {$mom_machine_mode == "PUNCH"} {
  } elseif {$mom_machine_mode == "LASER"} {
  } elseif {$mom_machine_mode == "TORCH"} {
  } elseif {$mom_machine_mode == "WIRE"} {
  }
}


proc MOM_cutcom_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Cutcom command is activated.
#_______________________________________________________________________________
  global mom_cutcom_status   mom_cutcom_mode
  if {[info exists mom_cutcom_mode]} { set mom_cutcom_status $mom_cutcom_mode }
}

proc MOM_cutcom_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Cutcom command is deactivated.
#_______________________________________________________________________________
  global mom_cutcom_status
  global mom_cutcom_mode

  catch {unset mom_cutcom_mode}
  MOM_do_template cutcom_off
}

proc MOM_spindle_rpm {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is activated.
#_______________________________________________________________________________
  global mom_spindle_speed
  global spindle_speed ; set  spindle_speed  1
  MOM_do_template spindle_start
}

#_______________________________________________________________________________
proc MOM_spindle_css {} {
#_______________________________________________________________________________
  MOM_spindle_rpm
}

proc MOM_spindle_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Spindle command is deactivated.
#_______________________________________________________________________________
   global  mom_spindle_speed spindle_speed
   MOM_do_template torm_off
   MOM_do_template spindle_off
   set  spindle_speed  0
}

proc MOM_coolant_on {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is activated.
#_______________________________________________________________________________
 global mom_coolant_status mom_coolant_mode
 set mom_coolant_status $mom_coolant_mode
}

proc MOM_coolant_off {} {
#_______________________________________________________________________________
# This procedure is executed when the Coolant command is deactivated.
#_______________________________________________________________________________
  MOM_do_template torm_off
  MOM_do_template coolant_off
}

proc MOM_opstop {} {
#_______________________________________________________________________________
# This procedure is executed when the Optional Stop command is activated.
#_______________________________________________________________________________
  MOM_do_template torm_off
  MOM_do_template opstop
}

#=============================================================
proc  MOM_operator_message {} {
#=============================================================
#__This procedure is executed when the Operator Message command is activated.
 global mom_operator_message  mom_operator_message_defined
 global mom_warning_info

  if {[info exists mom_operator_message]} {
    if {$mom_operator_message!=""} {
     #MOM_suppress once N
     MOM_output_literal "(MSG,'$mom_operator_message')"
    }
  }

}

proc MOM_stop {} {
#_______________________________________________________________________________
# This procedure is executed when the Program Stop command is activated.
#_______________________________________________________________________________
         MOM_do_template torm_off
         MOM_do_template stop
}

proc MOM_delay {} {
#_______________________________________________________________________________
# This procedure is executed when the Delay command is activated.
#_______________________________________________________________________________
         MOM_do_template delay
         MOM_do_template delay_1
}

proc MOM_auxfun {} {
#_______________________________________________________________________________
# This procedure is executed when the Auxiliary command is activated.
#_______________________________________________________________________________
  MOM_do_template auxiliary
}

proc MOM_prefun {} {
#_______________________________________________________________________________
# This procedure is executed when the Preparatory function command is activated.
#_______________________________________________________________________________
  MOM_do_template preparatory
}

proc MOM_length_compensation {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Length Compensation command is activated.
#_______________________________________________________________________________
}

proc MOM_tool_preselect {} {
#_______________________________________________________________________________
# This procedure is executed when the Tool Preselect command is activated.
#_______________________________________________________________________________
}

proc MOM_text {} {
#_______________________________________________________________________________
# This procedure is executed when the Text command is activated.
#_______________________________________________________________________________
  global mom_user_defined_text mom_record_text mom_record_fields mom_field_count
  global mom_Instruction
  global mom_sys_control_out mom_sys_control_in
  global mom_operator_message mom_operator_message_defined
  global mom_pprint_defined mom_pprint
  global mom_warning_info

  switch [string toupper $mom_record_fields(0)] {
  "SET" {  ;  }
  "PPRINT" {
           set mom_pprint_defined 1
           set mom_pprint $mom_record_text
           MOM_pprint
           }
  "DISPLY" {
           set mom_operator_message_defined 1
           set mom_operator_message $mom_record_text
           MOM_operator_message
          }
  "INSERT" {
           MOM_output_literal "$mom_record_fields(1)"
         }
  "MIRROR" {  ; #  
  		catch { PB_mirror }
        }
  "SAFETY" { ; #       
         }
  "SLOWDN" { ; }
  "TIME" { ; }
  "ZERO" {
           global  mom_work_coordinate_number
           set mom_work_coordinate_number $mom_record_fields(1)
           MOM_zero
         }
   default {
            set mom_warning_info "WARNING:   !($mom_user_defined_text)"
            MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
            MOM_output_literal $mom_user_defined_text
         }
  }
}


#=============================================================
proc  MOM_zero {} {
#=============================================================
#     G      .
#         Fanuc.
#   G   ZERO/CHECK, ZERO/ON, ZERO/OFF, ZERO/THRU,   G   ZERO/n.
#
  global mom_sys_wcs_change_code mom_work_coordinate_number
  if {[info exists mom_work_coordinate_number]==0} { set mom_work_coordinate_number 0 ; }
  if {$mom_work_coordinate_number==""} { set mom_work_coordinate_number 0 }
  MOM_output_literal "(UAO,$mom_work_coordinate_number)"

}


#=============================================================
proc  MOM_before_output {} {
#=============================================================
# This procedure is executed just before a line is about to be output
# to the file. It loads the line into a variable MOM_o_buffer and then calls
# this procedure. When it returns from this procedure, the variable MOM_o_buffer
# is read and written to the file.
#=============================================================
#########The following procedure invokes the listing file with warnings.
         LIST_FILE
# If the variable buffer_action is set to one then every string that comes
# through here will be buffered into an array of strings "buffer" and the
# mom_o_buffer is reset to an empty string so that no output is done.
 global mom_o_buffer

 if {$mom_o_buffer!=""} {

  set str $mom_o_buffer
  set var0 ""

  #     
  if {[string first $str ";" ]==-1} {
    set var0 ""
    # -  -  X Y Z - if 
    foreach i { X Y Z I J K } {

     #   i ------
     #set strv "($i)(\[- - +\])*(\[0-9\])*(\[.\])*(\[0-9\])+|($i)(\[- - +\])*(\[0-9\])+"
     set strv "($i)(\[- - +\])*(\[0-9\])*(\[.\])+(\[0-9\])*"
     set st [regexp -nocase -- "$strv"  $str  var0 ]
     if {$st!=0} {
       set var10 ""; set var9  "";
       scan $var0 "%1s %f" var9 var10
       if  { $var10!= ""} {
         set str_real $var10
         set str_int [ expr int($str_real) ]
         if {[EQ_is_equal $str_real $str_int]} {
         	set fl [regsub -nocase  -- "$strv"  $str  "$i$str_int"  var0 ]
         	set str $var0
         }
       }

    } ; #    i ----------

  } ; # foreach

   set mom_o_buffer $str
  } ; # string first

 }

  catch { PB_before_output }

}
